
-- this code manages the orientations of the tiles
-- should not to be confused with patterns.e which deals more with the layout

integer orient orient = 0 -- init
integer mod mod = 0 -- init
sequence text text = "" -- init

-- "orient" points to the current orientation style
-- "mod" means the rotation offset - 0 none, 1 90, 2 180, 3 270

---- combinations
-----------------
--
---- SQUARE // 7 states
--aligned
--rolling
--stagger
--nonadj - pivot
--		 - antipivot
--		 - other
--random
--
---- RECTANGLE
--
----stack/block
--aligned
--alternate
--stagger
--
---- recuadro

function isOdd(integer n)
	return remainder(n,2)
	end function

function isEven(integer n)
	return not remainder(n,2)
	end function

global function getOrientDesc()
	return text
	end function

global procedure resetOrientation()
	orient = 1
	mod = 0
	end procedure


global procedure changeOrientations(integer change)
	integer layout, num

	layout = getPatternLayout()

	if not layout then
		return 
	end if

	-- move pointer
	orient += change

	----------------------------------------------------------------------
	-- determine how many tile orientations exist for each layout pattern
	----------------------------------------------------------------------
	
	-- SQUARE
	if getTileKind() = TILE_SQUARE then
		num = 8

	-- RECTANGLE
	elsif getTileKind() = TILE_RECTANGLE then

		if layout = PATTERN_STACK or layout = PATTERN_BLOCK then
			num = 4

		elsif layout = PATTERN_RECUADRO or layout = PATTERN_CRISSCROSS then
			num = 16
	
		else
			?9/0

		end if

	else
		?9/0

	end if

	-- clip
	if orient > num then
		orient = 1
	elsif orient < 1 then
		orient = num
	end if

	-- set mod to NULL
	mod = 0

	end procedure



global procedure changeOrientation_fwd()
	mod += 1
	if mod > 3 then mod = 0 end if
	end procedure

global procedure changeOrientation_bak()
	mod -= 1
	if mod < 0 then mod = 3 end if
	end procedure


-------------------------------

constant
	NONADJODD = {1,3,2,4},
	NONADJEVE = {2,4,1,3}


function getSquare()
	sequence base, pattern, line, lines
	integer tmp

	-- make a nested sequence of each line
	pattern = getPattern()
	lines = {}
	line = {0}
	tmp = pattern[1][2] -- is now the Y coord of the 1st tile
	for i = 2 to length(pattern) do
		if pattern[i][2] != tmp then
			tmp = pattern[i][2]
			lines = append(lines, line)
			line = {}
		end if
		line &= 0
	end for
	lines = append(lines, line)

	----------------------------
	-- make orientation patterns
	----------------------------

	base = repeat(0, length(pattern))

	if orient = 1 then -- aligned
		text = "Same Alignment"
		base[1..$] = 1

	elsif orient = 2 then
		text = "Rolling Tile"
		tmp = 1
		base[1] = 1
		for i = 1 to length(lines) do
			for j = 1 to length(lines[i]) do
				base[tmp] = i-1+j
				tmp += 1
			end for
		end for

	elsif orient = 3 then
		text = "Staggered quarter turn"
		tmp = 1
		for i = 1 to length(lines) do
			for j = 1 to length(lines[i]) do
				base[tmp] = remainder(i+j,2)+1
				tmp += 1
			end for
		end for


	elsif orient = 4 then
		text = "Alternate lines 90 degrees"
		tmp = 1
		for i = 1 to length(lines) do
			for j = 1 to length(lines[i]) do
				if isOdd(i) then
					base[tmp] = 1
				else
					base[tmp] = 2
				end if
				tmp += 1
			end for
		end for

	elsif orient = 5 then
		text = "Alternate lines 180 degrees"
		tmp = 1
		for i = 1 to length(lines) do
			for j = 1 to length(lines[i]) do
				if isOdd(i) then
					base[tmp] = 1
				else
					base[tmp] = 3
				end if
				tmp += 1
			end for
		end for
		

	elsif orient = 6 then -- nonadjacent
		text = "Non-adjacent #1"
		tmp = 1
		for i = 1 to length(lines) do
			for j = 1 to length(lines[i]) do
				if isOdd(i) then
					base[tmp] = NONADJODD[remainder(j,4)+1]
				else
					base[tmp] = NONADJEVE[remainder(j,4)+1]
				end if
				tmp += 1
			end for
		end for


	elsif orient = 7 then -- nonadj - anticlockwise
		text = "Non-adjacent #2"
		tmp = 1
		for i = 1 to length(lines) do
			for j = 1 to length(lines[i]) do
				if isOdd(i) then
					base[tmp] = isEven(j)*3+1 -- {1,4,1,4..}
				else
					base[tmp] = isEven(j)+2 -- {2,3,2,3..}
				end if
				tmp += 1
			end for
		end for

	elsif orient = 8 then -- random
		text = "Random"
		base = rand(repeat(4, length(pattern)))

	else -- only supposed to be 7 !!
		?9/0

	end if

	--------------
	-- add offsets
	--------------
	base += mod
	for i = 1 to length(base) do
		base[i] = remainder(base[i], 4) + 1
	end for

	-- exit
	return base

	end function


function getRectangle()

	integer layout, tmp
	sequence base, rBits, pattern, line, lines

	--return getOrients()

	layout = getPatternLayout()

	if layout = PATTERN_STACK or layout = PATTERN_BLOCK then
		-- same
		-- alternate
		-- 180 deg
		-- 180 deg stagger
		
--		return getOrients()
-------------------------

		-- make a nested sequence of each line
		pattern = getPattern()
		lines = {}
		line = {0}
		tmp = pattern[1][2] -- is now the Y coord of the 1st tile
		for i = 2 to length(pattern) do
			if pattern[i][2] != tmp then
				tmp = pattern[i][2]
				lines = append(lines, line)
				line = {}
			end if
			line &= 0
		end for
		lines = append(lines, line)

		-- init
		base = repeat(0, length(pattern))

		if orient = 1 then
			text = "Same alignment"
			base[1..$] = 1

		elsif orient = 2 then
			text = "alternate lines"
			tmp = 1
			for i = 1 to length(lines) do
				for j = 1 to length(lines[i]) do
					if isOdd(i) then
						base[tmp] = 1
					else
						base[tmp] = 3
					end if
					tmp += 1
				end for
			end for


		elsif orient = 3 then
			text = "Loggerhead"
			tmp = 1
			for i = 1 to length(lines) do
				for j = 1 to length(lines[i]) do
					if isOdd(j) then
						base[tmp] = 1
					else
						base[tmp] = 3
					end if
					tmp += 1
				end for
			end for

		elsif orient = 4 then
			text = "Loggerhead stagger"
			tmp = 1
			for i = 1 to length(lines) do
				for j = 1 to length(lines[i]) do
					if isOdd(i) then
						base[tmp] = 1 + isOdd(j)*2
					else
						base[tmp] = 1 + isEven(j)*2
					end if
					tmp += 1
				end for
			end for

		else
			?9/0

		end if
		


	elsif layout = PATTERN_RECUADRO or layout = PATTERN_CRISSCROSS then

		if layout = PATTERN_RECUADRO then
			text = "Recuadro #"
		else
			text = "Crisscross #"
		end if
		text &= sprintf("%d", orient)
		-- just go through all the combinations
		base = getOrients()
		rBits = int_to_bits(orient-1,4) -- orient will be 0 to 15 and result is 4 bits long
		for i = 1 to 4 do
			if rBits[i] = 0 then
				rBits[i] = i
			elsif rBits[i] = 1 then
				rBits[i] = i+2
			end if
		end for
		rBits -= (rBits > 4) * 4
		for i = 1 to length(base) do
			base[i] = rBits[ base[i] ]
		end for

	else
		?9/0

	end if

	return base	
		
	end function


global function getOrientations()
	text = ""
	if getTileKind() = TILE_SQUARE then
		return getSquare()

	else
		return getRectangle()

	end if

	end function