

-- manages the various patterns that will be drawn etc..

sequence
	coords,
	orients -- 1 - 4

integer MAXFLOORWIDTH, MAXFLOORHEIGHT

integer layout layout = 0

global constant

	PATTERN_STACK = 1,
	PATTERN_BLOCK = 2,
	PATTERN_RECUADRO = 3,
	PATTERN_CRISSCROSS = 4

constant
	PATTERN_DESCRIPTION = { "None", "Stack", "Block", "Recuadro", "Crisscross" }

integer allow_over_bounds allow_over_bounds = 1

global function getPatternLayout()
	return layout
	end function

global function getPatternLayoutDesc()
	return PATTERN_DESCRIPTION[layout+1]
	end function

global function getPattern()
	return coords
	end function

global function getOrients()
	return orients
	end function

function isOdd(integer n)
	return and_bits(n,1)
	end function

function isEven(integer n)
	return not and_bits(n,1)
	end function

function isInFloor(integer x, integer y)
	if x >= 0 then
		if x <= MAXFLOORWIDTH then
			if y >= 0 then
				if y <= MAXFLOORHEIGHT then
					return 1
				end if
			end if
		end if
	end if
	return 0
	end function


function isAnyIn(integer x, integer y, integer width, integer height)
	if isInFloor(x, y) or
		isInFloor(x+width-1, y) or
		isInFloor(x+width-1, y+height-1) or
		isInFloor(x, y+height-1) then
		return 1
	end if
	return 0
	end function


function isAllIn(integer x, integer y, integer width, integer height)
	if isInFloor(x, y) and
		isInFloor(x+width-1, y) and
		isInFloor(x+width-1, y+height-1) and
		isInFloor(x, y+height-1) then
		return 1
	end if
	return 0
	end function	


procedure setFloorBounds(integer width, integer height)
	MAXFLOORWIDTH = width
	MAXFLOORHEIGHT	= height
	end procedure


procedure tryAddToCoords(integer x, integer y, integer width, integer height, integer orient)

	if allow_over_bounds then
		if isAnyIn(x, y, width, height) then
			coords = append(coords, {x,y})
			orients &= orient
		end if

	else
		if isAllIn(x, y, width, height) then
			coords = append(coords, {x,y})
			orients &= orient
		end if

	end if

	end procedure
	

global procedure changePattern(integer max_width, integer max_height, integer style)

	integer x, y, n, a, b
	integer size, kind

	kind = getTileKind() -- TILE_SQUARE or TILE_RECTANGLE
	size = getTileSize()

	-- reset
	coords = {}
	orients = {}

	-- set bounds
	setFloorBounds(max_width-1, max_height-1)

	-- slightly overestimate how many blocks are needed for each pattern
	x = floor(MAXFLOORWIDTH/size)+1
	y = floor(MAXFLOORHEIGHT/size)+1
		
	-- TILE_SQUARE
	if kind = TILE_SQUARE then

		if style = PATTERN_STACK then
			for i = 0 to y do
				for j = 0 to x do
					tryAddToCoords(j*size, i*size, size, size, 1 )
				end for
			end for

		elsif style = PATTERN_BLOCK then
			for i = 0 to y do
				for j = 0 to x do
					tryAddToCoords(j*size-(floor(size/2)*isOdd(i)), i*size, size, size, 1 )
				end for
			end for

		else
			?9/0 -- Error!!

		end if


	-- TILE_RECTANGLE
	elsif kind = TILE_RECTANGLE then

		if style = PATTERN_STACK then
			for i = 0 to y do
				for j = 0 to floor(x/2) + 1 do
					tryAddToCoords(j*size*2, i*size, size*2, size, 1 )
				end for
			end for

		elsif style = PATTERN_BLOCK then
			for i = 0 to y do
				for j = 0 to floor(x/2) + 1 do
					tryAddToCoords(j*size*2-(size*isOdd(i)), i*size, size*2, size, 1 )
				end for
			end for

		elsif style = PATTERN_RECUADRO then
			n = 0
			for i = 0 to y+1 by 2 do
				if not and_bits(i,2) then -- 00
					for j = 0 to x by 4 do
						tryAddToCoords(j*size, i*size, size*2, size, 1 )
						tryAddToCoords(j*size, (i+1)*size, size*2, size, 3 )
						tryAddToCoords((j+2)*size, i*size, size, size*2, 2 )
						tryAddToCoords((j+3)*size, i*size, size, size*2, 4 )
					end for
				else
					for j = 0 to x by 4 do
						tryAddToCoords(j*size, i*size, size, size*2, 2 )
						tryAddToCoords((j+1)*size, i*size, size, size*2, 4 )
						tryAddToCoords((j+2)*size, i*size, size*2, size, 1 )
						tryAddToCoords((j+2)*size, (i+1)*size, size*2, size, 3 )
					end for
				end if
			end for


		elsif style = PATTERN_CRISSCROSS then

			for i = 0 to y by 4 do

				for j = 0 to x by 4 do

					tryAddToCoords(j-1, i+2, size*2, size,   1 )
					tryAddToCoords(j,   i,   size,   size*2, 2 )
					tryAddToCoords(j+1, i,   size*2, size,   1 )
					tryAddToCoords(j+3, i-1, size,   size*2, 4 )

					tryAddToCoords(j,   i+3, size*2, size,   3 )
					tryAddToCoords(j+1, i+1, size,   size*2, 4 )
					tryAddToCoords(j+2, i+1, size*2, size,   3 )
					tryAddToCoords(j+2, i+2, size,   size*2, 2 )

				end for

			end for

			coords *= size


		else
			?9/0 -- Error!!

		end if

	-- NO MORE TILE KINDS!!!
	else
		?9/0 -- Error!!


	end if

	-- important to set this one!!
	layout = style

--?coords
	end procedure