
-- this file manages the image of the tile that is loaded in

-- the different kinds of tiles
global constant
	TILE_NONE = 0,
	TILE_SQUARE = 1, -- SAME LENGTH AS WIDTH
	TILE_RECTANGLE = 2 -- WIDTH IS LONGER THAN WIDTH, NB: if tile is tall then will be rotated 90 deg

integer
	TileKind,
	TileSize -- refers to the shortest edge, so, rectangular tiles are {size*2, size}

	TileKind = 0

sequence PathName

integer BaseTileImage, TILE1, TILE2, TILE3, TILE4

BaseTileImage = 0
	
TILE1 = 0
TILE2 = 0
TILE3 = 0
TILE4 = 0

global function isTileLoaded()
	return BaseTileImage
	end function

function max(integer a, integer b)
	if b > a then
		return b
	end if
	return a
	end function

function min(integer a, integer b)
	if b < a then
		return b
	end if
	return a
	end function

global function getTileForOrientation(integer orient)
	-- orient is 1 to 4
	if orient = 1 then
		return TILE1
	elsif orient = 2 then
		return TILE2
	elsif orient = 3 then
		return TILE3
	elsif orient = 4 then
		return TILE4
	end if
	end function

global function getTileKind()
	return TileKind
	end function

global function getTileSize()
	return TileSize
	end function


global function getTileName()
	sequence name
	name = extractPathAndName(PathName)
	return name[2]
	end function

global function getPathName()
	return PathName
	end function

global function loadTileBitmap(sequence file)
	-- returns 1 for success, 0 for failure
	integer id, w, h, kind
	atom tmp

	-- attempt to load tile as dibsection
	id = loadDibImage(file)
	if not id then -- couldn't load file
		return 0
	end if

	-- get the tile dimensions
	w = getDibWidth(id)
	h = getDibHeight(id)

	-- deduce the tile kind
	tmp = max(w,h) / min(w,h) -- tmp will never be less than 1
	if tmp < 1.1 then
		kind = TILE_SQUARE
	elsif tmp > 1.5 and tmp < 3 then
		kind = TILE_RECTANGLE
	else -- tile wrong dimension
		id = deleteDib(id)
		return 0
	end if

	-- set name here, don't do earlier
	PathName = file

	-- need to rotate?
	if h > w then
		rotateDib90(id)
	end if

	-- set as new id
	BaseTileImage = id
	TileKind = kind

	return 1

	end function


function makeOrientedTile(integer image_id, integer topleft, integer bottomright, integer groutcolour)
	integer w, h, id

	-- get dims of src image
	w = getDibWidth(image_id)
	h = getDibHeight(image_id)

	-- create slightly larger bitmap
	id = createDib(w+topleft+bottomright, h+topleft+bottomright, 32)

	-- copy image over, offset as needed
	copyDibToDib(id, topleft, topleft, image_id)

	-- draw grout lines
	for i = 1 to topleft do
		drawDibLineVertical(id, i-1, groutcolour)
		drawDibLineHorizontal(id, i-1, groutcolour)
	end for
	for i = 1 to bottomright do
		drawDibLineVertical(id, getDibWidth(id)-i, groutcolour)
		drawDibLineHorizontal(id, getDibHeight(id)-i, groutcolour)
	end for

	-- exit
	return id

	end function


global function makeFourTileOrientations(integer size, integer groutsize, integer groutcolour)
	integer glt, gbr, id -- id is a temp bitmap
	atom dc
	integer small_width, small_height

	-- must have existing tile
	if not BaseTileImage then
		return 0
	end if

	-- remove any prev images
	if TILE1 then
		void = deleteDib(TILE1)
		void = deleteDib(TILE2)
		void = deleteDib(TILE3)
		void = deleteDib(TILE4)
		TILE1 = 0
		TILE2 = 0
		TILE3 = 0
		TILE4 = 0
	end if

	-- deduce the top/left and bottom/right grout lines
	if groutsize < 0 then groutsize = 0 end if
	if groutsize > 10 then groutsize = 10 end if
	glt = floor(groutsize/2)
	gbr = floor((groutsize+1)/2)

	-- calculate the reduced dims
	small_width = size+((TileKind=TILE_RECTANGLE)*size)-glt-gbr
	small_height = size-glt-gbr

	-- create the temp bitmap as a reduced image of eventual composite size
	id = createDib(small_width, small_height, 32) -- tile is prone

	-- draw/stretch the original image into it
	stretchDibToDib(id, 0, 0, small_width, small_height, BaseTileImage, 0, 0, getDibWidth(BaseTileImage), getDibHeight(BaseTileImage))
	
	--	prepare 4 tiles each oriented slightly differently
	TILE1 = makeOrientedTile(id, glt, gbr, groutcolour)

	rotateDib90(id)
	TILE2 = makeOrientedTile(id, glt, gbr, groutcolour)

	rotateDib90(id)
	TILE3 = makeOrientedTile(id, glt, gbr, groutcolour)

	rotateDib90(id)
	TILE4 = makeOrientedTile(id, glt, gbr, groutcolour)

	void = deleteDib(id)

	TileSize = size



	return 1

	end function